package swingworkerdemo.fp;

/**
 * The Functional Part of the demo:
 * a slow implementation of the Fibonacci function.
 *
 * @author Tom Verhoeff (TU/e)
 */
public class Calculation {

    /** Lower bound on arguments for method {@code fib}. */
    public static final int LOWER_BOUND = 0;

    /**
     * Upper bound on arguments for method {@code fib}.
     * fib(47) does not fit in a long;
     * this also provides some protection in case the calculation
     * is not run in a separate thread and cannot be aborted.
     */
    public static final int UPPER_BOUND = 45;

    /** Flag to abort the calculation. */
    private boolean abort;

    /**
     * Determines whether a given number is a valid argument for
     * method {@code fig}.
     *
     * @param n  the argument to validate
     * @return  {@code LOWER_BOUND <= n && n <= UPPER_BOUND}
     */
    public static boolean validArgument(final int n) {
        return LOWER_BOUND <= n && n <= UPPER_BOUND;
    }

    /**
     * Requests the calculation to be aborted (at a convenient moment).
     */
    public void abort() {
        this.abort = true;
    }

    /**
     * Returns the {@code n}-th Fibonacci number.
     *
     * @param n  the index, starting at 0
     * @return the {@code n}-th Fibonacci number
     * @pre   {@code validArgument(n)}
     * @modifies {@code this}
     */
    public long fib(final int n) {
        if (! validArgument(n)) {
            throw new IllegalArgumentException(
                    "fib(" + n + ") violates @pre: argument out of range");
        }

        this.abort = false;
        long result = -1;
        try {
            result = fibRecursive(n);
        } catch (InterruptedException e) {
            // aborted, ignore exception
        }
        return result;
    }

    /**
     * Returns the {@code n}-th Fibonacci number.
     * Inefficient implementation (exponential in {@code n}).
     * The calculation can be interrupted by setting {@code abort}.
     *
     * @param n  the index, starting at 0
     * @return the {@code}-th Fibonacci number
     * @pre   {@code validArgument(n)}
     * @modifies {@code this}
     */
    private long fibRecursive(final int n) throws InterruptedException {
        if (abort) {
            throw new InterruptedException("Aborted by user");
        }

        if (n <= 1) {
            return n;
        } else {
            return fibRecursive(n - 1) + fibRecursive(n - 2);
        }
    }

}
